# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for sending a diagnostic interrupt to an instance."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.instances import flags

DETAILED_HELP = {
    'brief':
        'Send a diagnostic interrupt to a virtual machine instance.',
    'DESCRIPTION':
        """\
          *{command}* is used to send a diagnostic interrupt to a running
          instance, which triggers special interrupt handling logic inside VM.

        For instances with Intel or AMD processors, the guest OS on the instance
        will receive a non-maskable interrupt (NMI).
        """,
    'EXAMPLES':
        """\
        To send a diagnostic interrupt to an instance named ``test-instance'', run:

          $ {command} test-instance
        """
}


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class SendDiagnosticInterrupt(base.SilentCommand):
  """Send a diagnostic interrupt to a virtual machine instance."""

  @staticmethod
  def Args(parser):
    flags.INSTANCE_ARG.AddArgument(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    instance_ref = flags.INSTANCE_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=flags.GetInstanceZoneScopeLister(client))

    request = client.messages.ComputeInstancesSendDiagnosticInterruptRequest(
        **instance_ref.AsDict())

    return client.MakeRequests([(client.apitools_client.instances,
                                 'SendDiagnosticInterrupt', request)])


SendDiagnosticInterrupt.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shared resource flags for Cloud Monitoring commands."""


from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.monitoring import completers
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.command_lib.util.args import repeated
from googlecloudsdk.core.util import times


COMPARISON_TO_ENUM = {
    '>': 'COMPARISON_GT',
    '<': 'COMPARISON_LT',
    '>=': 'COMPARISON_GE',
    '<=': 'COMPARISON_LE',
    '==': 'COMPARISON_EQ',
    '=': 'COMPARISON_EQ',
    '!=': 'COMPARISON_NE',
}

UPTIME_MONITORED_RESOURCES = {
    'uptime-url': 'Uptime check against a URL.',
    'gce-instance': 'Uptime check against a Compute Engine instance.',
    'gae-app': 'Uptime check against an App Engine module.',
    'aws-ec2-instance': 'Uptime check against an AWS EC2 instance.',
    'aws-elb-load-balancer': 'Uptime check against an ElasticLoadBalancer.',
    'servicedirectory-service': (
        'Uptime check against a Service Directory service.'
    ),
    'cloud-run-revision': 'Uptime check against a Cloud Run revision.',
}

UPTIME_GROUP_RESOURCES = {
    'gce-instance': (
        'Uptime check against a group of instances from Google Cloud'
        ' or Amazon Web Services.'
    ),
    'aws-elb-load-balancer': (
        'Uptime check against a group of Amazon ELB load balancers.'
    ),
}

UPTIME_PROTOCOLS = {
    'http': 'An HTTP check.',
    'https': 'An HTTPS check.',
    'tcp': 'A TCP check.',
}

UPTIME_REQUEST_METHODS = {
    'get': 'HTTP GET method',
    'post': 'HTTP POST method',
}

UPTIME_CONTENT_TYPES = {
    'unspecified': 'Not specified',
    'url-encoded': 'URL encoded',
    'user-provided': 'User provided',
}

UPTIME_STATUS_CLASSES = {
    '1xx': 'Any response code from 100-199 inclusive',
    '2xx': 'Any response code from 200-299 inclusive',
    '3xx': 'Any response code from 300-399 inclusive',
    '4xx': 'Any response code from 400-499 inclusive',
    '5xx': 'Any response code from 500-599 inclusive',
    'any': 'Any response code',
}

UPTIME_PERIODS = {
    '1': 'One minute',
    '5': 'Five minutes',
    '10': 'Ten minutes',
    '15': 'Fifteen minutes',
}

UPTIME_REGIONS = {
    'usa-oregon': 'us-west1',
    'usa-virginia': 'us-east4',
    'usa-iowa': 'us-central1',
    'europe': 'europe-west1',
    'south-america': 'southamerica-east1',
    'asia-pacific': 'asia-southeast1',
}

UPTIME_MATCHER_TYPES = {
    'contains-string': 'Response contains string',
    'not-contains-string': 'Response does not contain string',
    'matches-regex': 'Response matches regex',
    'not-matches-regex': 'Response does not match regex',
    'matches-json-path': 'Response matches at JSONPath',
    'not-matches-json-path': 'Response does not match at JSONPath',
}

UPTIME_JSON_MATCHER_TYPES = {
    'exact-match': 'Response matches exact string at JSONPath',
    'regex-match': 'Response matches regex at JSONPath',
}

UPTIME_SERVICE_AGENT_TYPES = {
    'oidc-token': 'OIDC Token authentication',
}


def AddFileMessageFlag(parser, resource, flag=None):
  """Adds flags for specifying a message as a file to the parser."""
  parser.add_argument(
      '--{}-from-file'.format(flag or resource),
      type=arg_parsers.FileContents(),
      help='The path to a JSON or YAML file containing the {}.'.format(
          resource))


def AddMessageFlags(parser, resource, flag=None):
  """Adds flags for specifying a message as a string/file to the parser."""
  message_group = parser.add_group(mutex=True)
  message_group.add_argument(
      '--{}'.format(flag or resource),
      help='The {} as a string. In either JSON or YAML format.'.format(
          resource))
  message_group.add_argument(
      '--{}-from-file'.format(flag or resource),
      type=arg_parsers.FileContents(),
      help='The path to a JSON or YAML file containing the {}.'.format(
          resource))


def AddDisplayNameFlag(parser, resource, positional=False):
  if positional:
    base.Argument(
        'display_name',
        metavar='DISPLAY_NAME',
        help='Display name for the uptime check or synthetic monitor.',
    ).AddToParser(parser)
  else:
    parser.add_argument(
        '--display-name', help='The display name for the {}.'.format(resource)
    )


def AddCombinerFlag(parser, resource):
  """Adds flags for specifying a combiner, which defines how to combine the results of multiple conditions."""
  parser.add_argument(
      '--combiner',
      choices={
          'COMBINE_UNSPECIFIED': 'An unspecified combiner',
          'AND': 'An incident is created only if '
                 'all conditions are met simultaneously. '
                 'This combiner is satisfied if all conditions are met, '
                 'even if they are met on completely different resources.',
          'OR': 'An incident is created if '
                'any of the listed conditions is met.',
          'AND_WITH_MATCHING_RESOURCE': 'Combine conditions using '
                                        'logical AND operator, '
                                        'but unlike the regular AND option, '
                                        'an incident is created '
                                        'only if all conditions '
                                        'are met simultaneously '
                                        'on at least one resource.',
      },
      help='The combiner for the {}.'.format(resource))


def AddPolicySettingsFlags(parser, update=False):
  """Adds policy settings flags to the parser."""
  policy_settings_group = parser.add_group(help="""\
      Policy Settings.
      If any of these are specified, they will overwrite fields in the
      `--policy` or `--policy-from-file` flags if specified.""")
  AddDisplayNameFlag(policy_settings_group, resource='Alert Policy')
  AddCombinerFlag(policy_settings_group, resource='Alert Policy')
  enabled_kwargs = {
      'action': arg_parsers.StoreTrueFalseAction if update else 'store_true'
  }
  if not update:
    # Can't specify default if using StoreTrueFalseAction.
    enabled_kwargs['default'] = True
  policy_settings_group.add_argument(
      '--enabled', help='If the policy is enabled.', **enabled_kwargs)

  documentation_group = policy_settings_group.add_group(help='Documentation')
  documentation_group.add_argument(
      '--documentation-format',
      default='text/markdown' if not update else None,
      help='The MIME type that should be used with `--documentation` or '
           '`--documentation-from-file`. Currently, only "text/markdown" is '
           'supported.')
  documentation_string_group = documentation_group.add_group(mutex=True)
  documentation_string_group.add_argument(
      '--documentation',
      help='The documentation to be included with the policy.')
  documentation_string_group.add_argument(
      '--documentation-from-file',
      type=arg_parsers.FileContents(),
      help='The path to a file containing the documentation to be included '
           'with the policy.')
  if update:
    repeated.AddPrimitiveArgs(
        policy_settings_group,
        'Alert Policy',
        'notification-channels',
        'Notification Channels')
    AddUpdateLabelsFlags(
        'user-labels', policy_settings_group, group_text='User Labels')
  else:
    AddCreateLabelsFlag(policy_settings_group, 'user-labels', 'policy')


def AddFieldsFlagsWithMutuallyExclusiveSettings(parser,
                                                fields_help,
                                                add_settings_func,
                                                fields_choices=None,
                                                **kwargs):
  """Adds fields flags with mutually excludisve settings."""
  update_group = parser.add_group(mutex=True)
  update_group.add_argument(
      '--fields',
      metavar='field',
      type=arg_parsers.ArgList(choices=fields_choices),
      help=fields_help)
  add_settings_func(update_group, **kwargs)


def ValidateAlertPolicyUpdateArgs(args):
  """Validate alert policy update args."""
  if args.fields and not (args.policy or args.policy_from_file):
    raise exceptions.OneOfArgumentsRequiredException(
        ['--policy', '--policy-from-file'],
        'If --fields is specified.')


def ComparisonValidator(if_value):
  """Validates and returns the comparator and value."""
  if if_value.lower() == 'absent':
    return (None, None)
  if len(if_value) < 2:
    raise exceptions.BadArgumentException('--if', 'Invalid value for flag.')
  comparator_part = if_value[0]
  threshold_part = if_value[1:]
  try:
    comparator = COMPARISON_TO_ENUM[comparator_part]
    threshold_value = float(threshold_part)

    # currently only < and > are supported
    if comparator not in ['COMPARISON_LT', 'COMPARISON_GT']:
      raise exceptions.BadArgumentException('--if',
                                            'Comparator must be < or >.')
    return comparator, threshold_value
  except KeyError:
    raise exceptions.BadArgumentException('--if',
                                          'Comparator must be < or >.')
  except ValueError:
    raise exceptions.BadArgumentException('--if',
                                          'Threshold not a value float.')


def AddConditionSettingsFlags(parser):
  """Adds policy condition flags to the parser."""
  condition_group = parser.add_group(help="""\
        Condition Settings.
        This will add a condition to the created policy. If any conditions are
        already specified, this condition will be appended.""")
  condition_group.add_argument(
      '--condition-display-name',
      help='The display name for the condition.')
  condition_group.add_argument(
      '--condition-filter',
      help='Specifies the "filter" in a metric absence or metric threshold '
           'condition.')
  condition_group.add_argument(
      '--aggregation',
      help='Specifies an Aggregation message as a JSON/YAML value to be '
           'applied to the condition. For more information about the format: '
           'https://cloud.google.com/monitoring/api/ref_v3/rest/v3/'
           'projects.alertPolicies')
  condition_group.add_argument(
      '--duration',
      type=arg_parsers.Duration(),
      help='The duration (e.g. "60s", "2min", etc.) that the condition '
           'must hold in order to trigger as true.')
  AddUpdateableConditionFlags(condition_group)


def AddUpdateableConditionFlags(parser):
  """Adds flags for condition settings that are updateable to the parser."""
  parser.add_argument(
      '--if',
      dest='if_value',  # To avoid specifying args.if.
      type=ComparisonValidator,
      help='One of "absent", "< THRESHOLD", "> THRESHOLD" where "THRESHOLD" is '
           'an integer or float.')
  trigger_group = parser.add_group(mutex=True)
  trigger_group.add_argument(
      '--trigger-count',
      type=int,
      help='The absolute number of time series that must fail the predicate '
           'for the condition to be triggered.')
  trigger_group.add_argument(
      '--trigger-percent',
      type=float,
      help='The percentage of time series that must fail the predicate for '
           'the condition to be triggered.')


def ValidateNotificationChannelUpdateArgs(args):
  """Validate notification channel update args."""
  if (args.fields and
      not (args.channel_content or args.channel_content_from_file)):
    raise exceptions.OneOfArgumentsRequiredException(
        ['--channel-content', '--channel-content-from-file'],
        'If --fields is specified.')


def AddNotificationChannelSettingFlags(parser, update=False):
  """Adds flags for channel settings to the parser."""
  channel_group = parser.add_group(help='Notification channel settings')
  AddDisplayNameFlag(channel_group, 'channel')
  channel_group.add_argument(
      '--description',
      help='An optional description for the channel.')
  channel_group.add_argument(
      '--type',
      help='The type of the notification channel. This field matches the '
           'value of the NotificationChannelDescriptor type field.')

  enabled_kwargs = {
      'action': arg_parsers.StoreTrueFalseAction if update else 'store_true'
  }
  if not update:
    # Can't specify default if using StoreTrueFalseAction.
    enabled_kwargs['default'] = True
  channel_group.add_argument(
      '--enabled',
      help='Whether notifications are forwarded to the described channel.',
      **enabled_kwargs)
  if update:
    AddUpdateLabelsFlags(
        'user-labels', channel_group, group_text='User Labels')
    AddUpdateLabelsFlags(
        'channel-labels', channel_group, validate_values=False,
        group_text='Configuration Fields: Key-Value pairs that define the '
                   'channel and its behavior.')
  else:
    AddCreateLabelsFlag(channel_group, 'user-labels', 'channel')
    AddCreateLabelsFlag(
        channel_group, 'channel-labels', 'channel', validate_values=False,
        extra_message='These are configuration fields that define the channel '
                      'and its behavior.')


def AddCreateLabelsFlag(parser, labels_name, resource_name, extra_message='',
                        validate_values=True, skip_extra_message=False):
  """Add create labels flags."""
  if not skip_extra_message:
    extra_message += ('If the {0} was given as a JSON/YAML object from a '
                      'string or file, this flag will replace the labels value '
                      'in the given {0}.'.format(resource_name))
  labels_util.GetCreateLabelsFlag(
      extra_message=extra_message,
      labels_name=labels_name,
      validate_values=validate_values).AddToParser(parser)


def AddUpdateLabelsFlags(labels_name, parser, group_text='',
                         validate_values=True):
  """Add update labels flags."""
  labels_group = parser.add_group(group_text)
  labels_util.GetUpdateLabelsFlag(
      '', labels_name=labels_name,
      validate_values=validate_values).AddToParser(labels_group)
  remove_group = labels_group.add_group(mutex=True)
  labels_util.GetRemoveLabelsFlag(
      '', labels_name=labels_name).AddToParser(remove_group)
  labels_util.GetClearLabelsFlag(
      labels_name=labels_name).AddToParser(remove_group)


def GetMonitoredResourceContainerNameFlag(verb):
  """Flag for managing a monitored resource container."""
  return base.Argument(
      'monitored_resource_container_name',
      metavar='MONITORED_RESOURCE_CONTAINER_NAME',
      completer=completers.MonitoredResourceContainerCompleter,
      help=(
          'Monitored resource container (example - projects/PROJECT_ID) project'
          ' you want to {0}.'.format(verb)
      ),
  )


def AddCriteriaPoliciesFlag(parser, resource):
  parser.add_argument(
      '--criteria-policies',
      metavar='CRITERIA_POLICIES',
      type=arg_parsers.ArgList(min_length=1, max_length=16),
      help=(
          'The policies that the {} applies to. Exactly 1 alert policy is'
          ' required if `criteria-filter` is specified at the same time.'
          .format(resource)
      ),
  )


def AddCriteriaFilterFlag(parser, resource):
  parser.add_argument(
      '--criteria-filter',
      metavar='CRITERIA_FILTER',
      type=str,
      help=(
          'Optional. When you define a {}, you can also define a filter for'
          ' that snooze. The filter is a string containing one or more'
          ' key-value pairs. The string uses the standard'
          ' https://google.aip.dev/160 filter syntax. If you define a filter'
          ' for a snooze, then the snooze can only apply to one alert policy.'
          " When the snooze is active, incidents won't be created when the"
          ' incident would have key-value pairs (labels) that match those'
          ' specified by the filter in the snooze. Snooze filters support'
          ' resource, metric, and metadata labels. If multiple labels are used,'
          ' then they must be connected with an AND operator. For example:'
          ' resource.labels.instance_id="1234567890" AND'
          ' resource.labels.zone="us-central1-a" AND'
          ' metric.labels.instance_name="test_group" AND'
          ' metadata.user_labels.foo="bar" AND'
          ' metadata.system_labels.region="us-central1"'.format(resource)
      ),
  )


def AddStartTimeFlag(parser, resource):
  parser.add_argument(
      '--start-time',
      type=arg_parsers.Datetime.Parse,
      help='The start time for the {}.'.format(resource))


def AddEndTimeFlag(parser, resource):
  parser.add_argument(
      '--end-time',
      type=arg_parsers.Datetime.Parse,
      help='The end time for the {}.'.format(resource))


def AddSnoozeSettingsFlags(parser, update=False):
  """Adds snooze settings flags to the parser."""
  snooze_settings_group = parser.add_group(help="""\
      Snooze Settings.
      If any of these are specified, they will overwrite fields in the
      `--snooze-from-file` flags if specified.""")
  AddDisplayNameFlag(snooze_settings_group, resource='Snooze')
  if not update:
    AddCriteriaPoliciesFlag(snooze_settings_group, resource='Snooze')
    AddCriteriaFilterFlag(snooze_settings_group, resource='Snooze')
  AddStartTimeFlag(snooze_settings_group, resource='Snooze')
  AddEndTimeFlag(snooze_settings_group, resource='Snooze')


def AddUptimeSettingsFlags(parser, update=False):
  """Adds uptime check settings flags to the parser."""
  if not update:
    AddUptimeResourceFlags(parser)

  AddUptimeProtocolFlags(parser, update)
  AddUptimeRunFlags(parser, update)
  AddUptimeMatcherFlags(parser)


def AddUptimeResourceFlags(parser):
  """Adds uptime check resource settings flags to the parser."""
  uptime_resource_group = parser.add_group(
      help='Uptime check resource.',
      mutex=True,
      required=True,
  )
  monitored_resource_group = uptime_resource_group.add_group(
      help='Monitored resource'
  )
  monitored_resource_group.add_argument(
      '--resource-type',
      help='Type of monitored resource, defaults to `uptime-url`.',
      choices=UPTIME_MONITORED_RESOURCES,
  )
  base.Argument(
      '--resource-labels',
      metavar='KEY=VALUE',
      type=arg_parsers.ArgDict(key_type=str, value_type=str),
      action=arg_parsers.UpdateAction,
      required=True,
      help=(
          """Values for all of the labels listed in the associated monitored resource descriptor.
            See https://cloud.google.com/monitoring/api/resources for more information and allowed
            keys."""
      ),
  ).AddToParser(monitored_resource_group)
  group_resource_group = uptime_resource_group.add_group(
      help='Monitored resource group'
  )
  group_resource_group.add_argument(
      '--group-type',
      help=(
          'The resource type of the group members, defaults to `gce-instance`.'
      ),
      choices=UPTIME_GROUP_RESOURCES,
  )
  group_resource_group.add_argument(
      '--group-id',
      help='The group of resources being monitored.',
      required=True,
      type=str,
  )
  uptime_resource_group.add_argument(
      '--synthetic-target',
      help="""The target of the Synthetic Monitor.
        This is the fully qualified GCFv2 resource name.
        """,
      type=str,
  )


def AddUptimeProtocolFlags(parser, update=False):
  """Adds uptime check protocol settings flags to the parser."""
  uptime_protocol_group = parser.add_group(
      help='Uptime check protocol settings.'
  )
  if not update:
    uptime_protocol_group.add_argument(
        '--protocol',
        help='The protocol of the request, defaults to `http`.',
        choices=UPTIME_PROTOCOLS,
    )
  uptime_protocol_group.add_argument(
      '--port',
      help="""The port on the server against which to run the check.
        Defaults to `80` when `--protocol` is `http`.
        Defaults to `443` when `--protocol` is `https`.
        Required if `--protocol` is `tcp`.""",
      type=arg_parsers.BoundedInt(lower_bound=1, upper_bound=65535),
  )
  uptime_protocol_group.add_argument(
      '--pings-count',
      help='Number of ICMP pings to send alongside the request.',
      type=arg_parsers.BoundedInt(lower_bound=1, upper_bound=3),
  )
  uptime_protocol_group.add_argument(
      '--request-method',
      help="""The HTTP request method to use, defaults to `get`.
        Can only be set if `--protocol` is `http` or `https`.""",
      choices=UPTIME_REQUEST_METHODS,
  )
  uptime_protocol_group.add_argument(
      '--path',
      help="""The path to the page against which to run the check, defaults to `/`.
        Can only be set if `--protocol` is `http` or `https`.""",
      type=str,
  )
  uptime_protocol_group.add_argument(
      '--username',
      help="""The username to use when authenticating with the HTTP server.
        Can only be set if `--protocol` is `http` or `https`.""",
      type=str,
  )
  uptime_protocol_group.add_argument(
      '--password',
      help="""The password to use when authenticating with the HTTP server.
        Can only be set if `--protocol` is `http` or `https`.""",
      type=str,
  )
  uptime_protocol_group.add_argument(
      '--mask-headers',
      help="""Whether to encrypt the header information, defaults to `false`.
        Can only be set if `--protocol` is `http` or `https`.""",
      type=bool,
  )
  uptime_service_agent_auth_group = uptime_protocol_group.add_group(
      help='Uptime check service agent authorization.'
  )
  uptime_service_agent_auth_group.add_argument(
      '--service-agent-auth',
      help="""The type of authentication to use for the HTTP request.
        Can only be set if `--protocol` is `https`.""",
      choices=UPTIME_SERVICE_AGENT_TYPES)
  if update:
    uptime_headers_group = uptime_protocol_group.add_group(
        help='Uptime check headers.'
    )
    base.Argument(
        '--update-headers',
        metavar='KEY=VALUE',
        type=arg_parsers.ArgDict(key_type=str, value_type=str),
        action=arg_parsers.UpdateAction,
        help=("""The list of headers to add to the uptime check. Any existing
              headers with matching "key" are overridden by the provided
              values."""),
    ).AddToParser(uptime_headers_group)
    uptime_remove_header_group = uptime_headers_group.add_group(
        help='Uptime check remove headers.',
        mutex=True,
    )
    uptime_remove_header_group.add_argument(
        '--remove-headers',
        metavar='KEY',
        help="""The list of header keys to remove from the uptime check.""",
        type=arg_parsers.ArgList(str),
    )
    uptime_remove_header_group.add_argument(
        '--clear-headers',
        help="""Clear all headers on the uptime check.""",
        type=bool,
    )
  else:
    base.Argument(
        '--headers',
        metavar='KEY=VALUE',
        type=arg_parsers.ArgDict(key_type=str, value_type=str),
        action=arg_parsers.UpdateAction,
        help=(
            """The list of headers to send as part of the uptime check
              request. Can only be set if `--protocol` is `http` or `https`."""
        ),
    ).AddToParser(uptime_protocol_group)
  uptime_protocol_group.add_argument(
      '--content-type',
      help="""The content type header to use for the check, defaults to `unspecified`.
        Can only be set if `--protocol` is `http` or `https`.""",
      choices=UPTIME_CONTENT_TYPES,
  )
  uptime_protocol_group.add_argument(
      '--custom-content-type',
      help="""A user-provided content type header to use for the check.
        Can only be set if `--protocol` is `http` or `https`.""",
      type=str,
  )
  uptime_protocol_group.add_argument(
      '--validate-ssl',
      help="""Whether to include SSL certificate validation as a part of the uptime check,
        defaults to `false`.
        Can only be set if `--protocol` is `http` or `https`.""",
      type=bool,
  )
  uptime_protocol_group.add_argument(
      '--body',
      help="""The request body associated with the HTTP POST request.
        Can only be set if `--protocol` is `http` or `https`.""",
      type=str,
  )
  uptime_status_group = uptime_protocol_group.add_group(
      help='Uptime check status.',
      mutex=True,
  )
  if update:
    uptime_status_classes_group = uptime_status_group.add_group(
        help='Uptime check status classes.',
        mutex=True,
    )
    uptime_status_classes_group.add_argument(
        '--set-status-classes',
        metavar='status-class',
        help="""List of HTTP status classes. The uptime check will only pass if the response
                code is contained in this list.""",
        type=arg_parsers.ArgList(choices=UPTIME_STATUS_CLASSES),
    )
    uptime_status_classes_group.add_argument(
        '--add-status-classes',
        metavar='status-class',
        help="""The list of HTTP status classes to add to the uptime check.""",
        type=arg_parsers.ArgList(choices=UPTIME_STATUS_CLASSES),
    )
    uptime_status_classes_group.add_argument(
        '--remove-status-classes',
        metavar='status-class',
        help="""The list of HTTP status classes to remove from the uptime check.""",
        type=arg_parsers.ArgList(choices=UPTIME_STATUS_CLASSES),
    )
    uptime_status_classes_group.add_argument(
        '--clear-status-classes',
        help="""Clear all HTTP status classes on the uptime check. Setting this
            flag is the same as selecting only the `2xx` status class.""",
        type=bool,
    )
    uptime_status_codes_group = uptime_status_group.add_group(
        help='Uptime check status codes.',
        mutex=True,
    )
    uptime_status_codes_group.add_argument(
        '--set-status-codes',
        metavar='status-code',
        help="""List of HTTP status codes. The uptime check will only pass if the response
                code is present in this list.""",
        type=arg_parsers.ArgList(int),
    )
    uptime_status_codes_group.add_argument(
        '--add-status-codes',
        metavar='status-code',
        help="""The list of HTTP status codes to add to the uptime check.""",
        type=arg_parsers.ArgList(int),
    )
    uptime_status_codes_group.add_argument(
        '--remove-status-codes',
        metavar='status-code',
        help="""The list of HTTP status codes to remove from the uptime check.""",
        type=arg_parsers.ArgList(int),
    )
    uptime_status_codes_group.add_argument(
        '--clear-status-codes',
        help="""Clear all HTTP status codes on the uptime check. Setting this
            flag is the same as selecting only the `2xx` status class.""",
        type=bool,
    )
  else:
    uptime_status_group.add_argument(
        '--status-classes',
        metavar='status-class',
        help="""List of HTTP status classes. The uptime check only passes when the response
              code is contained in this list. Defaults to `2xx`.
              Can only be set if `--protocol` is `http` or `https`.""",
        type=arg_parsers.ArgList(choices=UPTIME_STATUS_CLASSES),
    )
    uptime_status_group.add_argument(
        '--status-codes',
        metavar='status-code',
        help="""List of HTTP Status Codes. The uptime check will only pass if the response code
              is present in this list.
              Can only be set if `--protocol` is `http` or `https`.""",
        type=arg_parsers.ArgList(int),
    )


def AddUptimeRunFlags(parser, update=False):
  """Adds uptime check run flags to the parser."""
  uptime_settings_group = parser.add_group(help='Settings.')
  uptime_settings_group.add_argument(
      '--period',
      help='''The time between uptime check or synthetic monitor executions in
              minutes, defaults to `1`. Can be set for synthetic monitors.''',
      choices=UPTIME_PERIODS,
  )
  uptime_settings_group.add_argument(
      '--timeout',
      help=(
          'The maximum amount of time in seconds to wait for the request to'
          ' complete, defaults to `60`. Can be set for synthetic monitors.'
      ),
      type=arg_parsers.BoundedInt(lower_bound=1, upper_bound=60),
  )
  if update:
    AddDisplayNameFlag(
        uptime_settings_group,
        resource='uptime check or synthetic monitor',
        positional=False,
    )
    uptime_regions_group = uptime_settings_group.add_group(
        help="""Uptime check selected regions.""",
        mutex=True,
    )
    uptime_regions_group.add_argument(
        '--set-regions',
        metavar='region',
        help="""The list of regions from which the check is run. At least 3 regions must be
            selected.""",
        type=arg_parsers.ArgList(choices=UPTIME_REGIONS),
    )
    uptime_regions_group.add_argument(
        '--add-regions',
        metavar='region',
        help="""The list of regions to add to the uptime check.""",
        type=arg_parsers.ArgList(choices=UPTIME_REGIONS),
    )
    uptime_regions_group.add_argument(
        '--remove-regions',
        metavar='region',
        help="""The list of regions to remove from the uptime check.""",
        type=arg_parsers.ArgList(choices=UPTIME_REGIONS),
    )
    uptime_regions_group.add_argument(
        '--clear-regions',
        help="""Clear all regions on the uptime check. This setting acts the same as if all available
            regions were selected.""",
        type=bool,
    )
  else:
    uptime_settings_group.add_argument(
        '--regions',
        metavar='field',
        help="""The list of regions from which the check is run. At least 3 regions must be selected.
            Defaults to all available regions.""",
        type=arg_parsers.ArgList(choices=UPTIME_REGIONS),
    )
  if update:
    AddUpdateLabelsFlags(
        'user-labels',
        uptime_settings_group,
        'User labels. Can be set for synthetic monitors.',
    )
  else:
    AddCreateLabelsFlag(
        uptime_settings_group,
        'user-labels',
        'User labels. Can be set for synthetic monitors.',
        skip_extra_message=True,
    )


def AddUptimeMatcherFlags(parser):
  """Adds uptime check matcher flags to the parser."""
  uptime_matcher_group = parser.add_group(help='Uptime check matcher settings.')
  uptime_matcher_group.add_argument(
      '--matcher-content',
      required=True,
      type=str,
      help='String, regex or JSON content to match.',
  )
  uptime_matcher_group.add_argument(
      '--matcher-type',
      choices=UPTIME_MATCHER_TYPES,
      help="""The type of content matcher that is applied to the server output, defaults to
        `contains-string`.""",
  )
  uptime_json_matcher_group = uptime_matcher_group.add_group(
      help='Uptime check matcher settings for JSON responses.'
  )
  uptime_json_matcher_group.add_argument(
      '--json-path',
      type=str,
      required=True,
      help="""JSONPath within the response output pointing to the expected content to match.
            Only used if `--matcher-type` is `matches-json-path` or `not-matches-json-path`.""",
  )
  uptime_json_matcher_group.add_argument(
      '--json-path-matcher-type',
      choices=UPTIME_JSON_MATCHER_TYPES,
      help="""The type of JSONPath match that is applied to the JSON output, defaults to
            `exact-match`.
            Only used if `--matcher-type` is `matches-json-path` or `not-matches-json-path`.""",
  )


def ValidateSnoozeUpdateArgs(args):
  """Validate snooze update args."""
  if args.fields and not args.snooze_from_file:
    raise exceptions.OneOfArgumentsRequiredException(
        ['--snooze-from-file'], 'If --fields is specified.'
    )


def ValidateSnoozeInterval(
    snooze,
    display_name=None,
    start_time=None,
    end_time=None,
):
  """Validate snooze reference interval."""
  snooze_past = False
  end_time_ref = times.ParseDateTime(snooze.interval.endTime)
  if end_time_ref < times.Now():
    snooze_past = True

  if snooze_past:
    if display_name is not None:
      raise exceptions.InvalidArgumentException(
          '--display-name',
          'Expired snoozes can no longer be updated.',
      )
    elif start_time is not None:
      raise exceptions.InvalidArgumentException(
          '--start-time',
          'Expired snoozes can no longer be updated.',
      )
    elif end_time is not None:
      raise exceptions.InvalidArgumentException(
          '--end-time',
          'Expired snoozes can no longer be updated.',
      )


def AddMigrateFlags(parser):
  """Adds migrate flags to the parser."""
  migrate_group = parser.add_group()
  migrate_group.add_argument(
      '--policies-from-prometheus-alert-rules-yaml',
      metavar='PROMETHEUS_ALERT_RULE_FILE_PATHS',
      type=arg_parsers.ArgList(arg_parsers.FileContents()),
      help=(
          'One or more Prometheus alert rule YAML files (separated by commas if'
          ' multiple) to be converted to Cloud Alerting Policies. Example: '
          '--policies-from-prometheus-alert-rules-yaml=rules_1.yaml,rules_2.yaml'
      ),
  )
  migrate_group.add_argument(
      '--channels-from-prometheus-alertmanager-yaml',
      metavar='PROMETHEUS_ALERT_MANAGER_FILE_PATH',
      type=arg_parsers.FileContents(),
      help=(
          'Prometheus alert manager YAML file to be converted to Cloud'
          ' Monitoring notification channels. Specifying this flag with the'
          ' --policies-from-prometheus-alert-rules-yaml flag puts the newly'
          " created notification channels into the translated Alert Policies'"
          ' definition.'
      ),
  )



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Perform maintenance command."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.sole_tenancy.node_groups import flags
from googlecloudsdk.core import log
from googlecloudsdk.core.util import times


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class PerformMaintenance(base.UpdateCommand):
  """Perform maintenance on nodes in a Compute Engine node group."""

  detailed_help = {
      'brief': 'Perform maintenance on nodes in a Compute Engine node group.',
      'EXAMPLES': """
       To perform maintenance on nodes in a node group, run:

         $ {command} my-node-group --nodes=node-1,node-2 --start-time=2023-05-01T00:00:00.000-08:00
     """,
  }

  @staticmethod
  def Args(parser):
    flags.MakeNodeGroupArg().AddArgument(parser)
    flags.AddPerformMaintenanceNodesArgToParser(parser)
    flags.AddPerformMaintenanceStartTimeArgToParser(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    messages = holder.client.messages

    node_group_ref = flags.MakeNodeGroupArg().ResolveAsResource(
        args,
        holder.resources,
        scope_lister=compute_flags.GetDefaultScopeLister(holder.client))

    perform_maintenance = messages.NodeGroupsPerformMaintenanceRequest(
        nodes=args.nodes
    )

    if args.start_time:
      perform_maintenance.startTime = times.FormatDateTime(args.start_time)

    request = messages.ComputeNodeGroupsPerformMaintenanceRequest(
        nodeGroupsPerformMaintenanceRequest=perform_maintenance,
        nodeGroup=node_group_ref.Name(),
        project=node_group_ref.project,
        zone=node_group_ref.zone)

    service = holder.client.apitools_client.nodeGroups
    operation = service.PerformMaintenance(request)
    operation_ref = holder.resources.Parse(
        operation.selfLink, collection='compute.zoneOperations')
    log.status.Print(
        'Perform maintenance call in progress for nodes [{}] in node group'
        ' [{}]: {}'.format(
            args.nodes, node_group_ref.Name(), operation_ref.SelfLink()
        )
    )
    log.status.Print(
        'Use [gcloud compute operations describe URI] to check the status of'
        ' the operation(s).'
    )
    return operation



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for updating instances in a managed instance group."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import instance_groups_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.instance_groups import flags as instance_groups_flags
from googlecloudsdk.command_lib.compute.instance_groups.managed import flags as instance_groups_managed_flags
from googlecloudsdk.command_lib.compute.managed_instance_groups import update_instances_utils


@base.ReleaseTracks(
    base.ReleaseTrack.GA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.PREVIEW,
)
class UpdateInstances(base.Command):
  r"""Immediately update selected instances in a Compute Engine managed instance group."""

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat("""
        table(project(),
              zone(),
              instanceName:label=INSTANCE,
              status)""")
    instance_groups_managed_flags.AddUpdateInstancesArgs(parser=parser)
    instance_groups_flags.MULTISCOPE_INSTANCE_GROUP_MANAGER_ARG.AddArgument(
        parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    igm_ref = (instance_groups_flags.MULTISCOPE_INSTANCE_GROUP_MANAGER_ARG
               .ResolveAsResource)(
                   args,
                   holder.resources,
                   default_scope=compute_scope.ScopeEnum.ZONE,
                   scope_lister=flags.GetDefaultScopeLister(client))

    update_instances_utils.ValidateIgmReference(igm_ref)

    if igm_ref.Collection() == 'compute.instanceGroupManagers':
      minimal_action = update_instances_utils.ParseInstanceActionFlag(
          '--minimal-action', args.minimal_action or 'none',
          client.messages.InstanceGroupManagersApplyUpdatesRequest
          .MinimalActionValueValuesEnum)
      most_disruptive_allowed_action = (
          update_instances_utils.ParseInstanceActionFlag)(
              '--most-disruptive-allowed-action',
              args.most_disruptive_allowed_action or 'replace',
              client.messages.InstanceGroupManagersApplyUpdatesRequest
              .MostDisruptiveAllowedActionValueValuesEnum)
      instances_holder_field = 'instanceGroupManagersApplyUpdatesRequest'
      request = self._CreateZonalApplyUpdatesRequest(
          igm_ref, minimal_action, most_disruptive_allowed_action, client)
    elif igm_ref.Collection() == 'compute.regionInstanceGroupManagers':
      minimal_action = update_instances_utils.ParseInstanceActionFlag(
          '--minimal-action', args.minimal_action or 'none',
          client.messages.RegionInstanceGroupManagersApplyUpdatesRequest
          .MinimalActionValueValuesEnum)
      most_disruptive_allowed_action = (
          update_instances_utils.ParseInstanceActionFlag)(
              '--most-disruptive-allowed-action',
              args.most_disruptive_allowed_action or 'replace',
              client.messages.RegionInstanceGroupManagersApplyUpdatesRequest
              .MostDisruptiveAllowedActionValueValuesEnum)
      instances_holder_field = 'regionInstanceGroupManagersApplyUpdatesRequest'
      request = self._CreateRegionalApplyUpdatesRequest(
          igm_ref, minimal_action, most_disruptive_allowed_action, client)
    else:
      raise ValueError('Unknown reference type {0}'.format(
          igm_ref.Collection()))

    if args.all_instances:
      return instance_groups_utils.SendAllInstancesRequest(
          api_holder=holder,
          method_name='ApplyUpdatesToInstances',
          request_template=request,
          all_instances_holder_field=instances_holder_field,
          igm_ref=igm_ref)
    else:
      return instance_groups_utils.SendInstancesRequestsAndPostProcessOutputs(
          api_holder=holder,
          method_name='ApplyUpdatesToInstances',
          request_template=request,
          instances_holder_field=instances_holder_field,
          igm_ref=igm_ref,
          instances=args.instances)

  def _CreateZonalApplyUpdatesRequest(self, igm_ref, minimal_action,
                                      most_disruptive_allowed_action, client):
    return client.messages.ComputeInstanceGroupManagersApplyUpdatesToInstancesRequest(
        instanceGroupManager=igm_ref.Name(),
        instanceGroupManagersApplyUpdatesRequest=client.messages
        .InstanceGroupManagersApplyUpdatesRequest(
            instances=[],
            minimalAction=minimal_action,
            mostDisruptiveAllowedAction=most_disruptive_allowed_action),
        project=igm_ref.project,
        zone=igm_ref.zone)

  def _CreateRegionalApplyUpdatesRequest(self, igm_ref, minimal_action,
                                         most_disruptive_allowed_action,
                                         client):
    return client.messages.ComputeRegionInstanceGroupManagersApplyUpdatesToInstancesRequest(
        instanceGroupManager=igm_ref.Name(),
        regionInstanceGroupManagersApplyUpdatesRequest=client.messages
        .RegionInstanceGroupManagersApplyUpdatesRequest(
            instances=[],
            minimalAction=minimal_action,
            mostDisruptiveAllowedAction=most_disruptive_allowed_action),
        project=igm_ref.project,
        region=igm_ref.region,
    )


UpdateInstances.detailed_help = {
    'brief':
        'Immediately update selected instances in a Compute Engine '
        'managed instance group.',
    'DESCRIPTION':
        """\
          When using a managed instance group, it's possible that your intended
          specification for a VM is different from the current state of that VM. For
          example, this can happen due to changes to the group's target instance
          template. This command enables you to initiate the update process on the given
          set of instances instantly, thus when your Managed Instance Group is stable
          you can be sure that all the changes were applied.

          *{command}* allows you to specify the least and the most disruptive actions
          that can be performed while updating the instances. This way you can reduce
          the risk of rolling out too many changes at once. Possible actions are:
          `none`, `refresh`, `restart` and `replace`. The level of disruption to the
          instance is ordered as: `none` < `refresh` < `restart` < `replace`.

          The command returns the operation status per instance, which might be
          ``FAIL'', ``SUCCESS'', or ``MEMBER_NOT_FOUND''. ``MEMBER_NOT_FOUND''
          is returned only for regional groups when the gcloud command-line tool
          wasn't able to resolve the zone from the instance name.
        """,
    'EXAMPLES':
        """\
        To update instances `instance-1`, `instance-2` in `my-group`,
        with `minimal-action=none` and `most-disruptive-allowed-action=restart`,
        run:

            $ {command} \\
                  my-group --instances=instance-1,instance2 \\
                  --minimal-action=none
                  --most-disruptive-allowed-action=restart
        """
}



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for describing multi-MIGs."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.multi_migs import utils as api_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.multi_migs import utils as flags


DETAILED_HELP = {
    'EXAMPLES': """\
  To describe a multi-MIG with the name 'example-multimig', run:

    $ {command} example-multimig
  """,
}


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.BETA)
class DescribeBeta(base.DescribeCommand):
  """Describe a multi-MIG.

  *{command}* describes a multi-MIG.
  """

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    flags.AddMultiMigNameArgToParser(
        parser, base.ReleaseTrack.BETA.name.lower()
    )

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    multi_mig_ref = args.CONCEPTS.multi_mig.Parse()

    return api_utils.Get(client, multi_mig_ref)


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class DescribeAlpha(DescribeBeta):
  """Describe a multi-MIG."""

  @classmethod
  def Args(cls, parser):
    flags.AddMultiMigNameArgToParser(
        parser, base.ReleaseTrack.ALPHA.name.lower()
    )



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to list all Project IDs linked with a billing account."""


from googlecloudsdk.api_lib.billing import billing_client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.billing import flags
from googlecloudsdk.command_lib.billing import utils


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class ListAlpha(base.ListCommand):
  """List all active projects associated with the specified billing account.

  *{command}* ACCOUNT_ID -- lists all active projects, for the specified
  billing account id.

  ## EXAMPLES

  To list projects linked to billing account `0X0X0X-0X0X0X-0X0X0X`, run:

      $ {command} 0X0X0X-0X0X0X-0X0X0X

  ## API REFERENCE

  This command uses the *cloudbilling/v1* API. The full documentation for this
  API can be found at: https://cloud.google.com/billing/v1/getting-started
  """

  @staticmethod
  def Args(parser):
    account_args_group = parser.add_mutually_exclusive_group(required=True)
    flags.GetOldAccountIdArgument().AddToParser(account_args_group)
    flags.GetAccountIdArgument(positional=False).AddToParser(account_args_group)

    base.URI_FLAG.RemoveFromParser(parser)
    parser.display_info.AddFormat("""
          table(
            projectId,
            billingAccountName.basename():label=BILLING_ACCOUNT_ID,
            billingEnabled
          )
    """)
    parser.display_info.AddCacheUpdater(flags.BillingAccountsCompleter)

  @staticmethod
  def GetUriCacheUpdateOp():
    """No resource URIs."""
    return None

  def Run(self, args):
    """Run the list command."""
    client = billing_client.ProjectsClient()
    account_ref = utils.ParseAccount(args.id or args.billing_account)
    return client.List(account_ref, limit=args.limit)


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class List(base.ListCommand):
  """List all active projects associated with the specified billing account.

  *{command}* ACCOUNT_ID -- lists all active projects, for the specified
  billing account id.

  ## EXAMPLES

  To list projects linked to billing account `0X0X0X-0X0X0X-0X0X0X`, run:

      $ {command} --billing-account=0X0X0X-0X0X0X-0X0X0X
  """

  @staticmethod
  def Args(parser):
    flags.GetAccountIdArgument(positional=False,
                               required=True).AddToParser(parser)
    base.URI_FLAG.RemoveFromParser(parser)
    parser.display_info.AddFormat("""
          table(
            projectId,
            billingAccountName.basename():label=BILLING_ACCOUNT_ID,
            billingEnabled
          )
    """)

  @staticmethod
  def GetUriCacheUpdateOp():
    """No resource URIs."""
    return None

  def Run(self, args):
    """Run the list command."""
    client = billing_client.ProjectsClient()
    account_ref = utils.ParseAccount(args.billing_account)
    return client.List(account_ref, limit=args.limit)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command to create named configuration."""

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.config import config_validators
from googlecloudsdk.command_lib.projects import util as projects_util
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.configurations import named_configs
from googlecloudsdk.core.universe_descriptor import universe_descriptor


@base.UniverseCompatible
class Create(base.SilentCommand):
  """Creates a new named configuration."""

  detailed_help = {
      'DESCRIPTION': """\
          {description}

          Except for special cases (NONE), configuration names start with a
          lower case letter and contain only lower case letters a-z, digits 0-9,
          and hyphens '-'.

          See `gcloud topic configurations` for an overview of named
          configurations.
          """,
      'EXAMPLES': """\
          To create a new named configuration, run:

            $ {command} my-config
          """,
  }

  @staticmethod
  def Args(parser):
    """Adds args for this command."""
    parser.add_argument(
        'configuration_name', help='Name of the configuration to create'
    )
    parser.add_argument(
        '--activate',
        action='store_true',
        default=True,
        help='If true, activate this configuration upon create.',
    )
    parser.add_argument(
        '--universe-domain',
        type=str,
        hidden=True,
        help=(
            'If set, creates the configuration with the specified'
            ' [core/universe_domain].'
        ),
    )

  def Run(self, args):
    created_config = named_configs.ConfigurationStore.CreateConfig(
        args.configuration_name
    )
    log.CreatedResource(args.configuration_name)
    if args.activate:
      named_configs.ConfigurationStore.ActivateConfig(args.configuration_name)
      log.status.Print('Activated [{0}].'.format(args.configuration_name))
      project_id = properties.VALUES.core.project.Get()
      if project_id:
        projects_util.PrintEnvironmentTagMessageProjectId(
            project_id
        )
    else:
      log.status.Print(
          'To use this configuration, activate it by running:\n'
          '  $ gcloud config configurations activate {name}\n\n'.format(
              name=args.configuration_name
          )
      )
    if args.universe_domain:
      config_validators.WarnIfSettingUniverseDomainWithNoDescriptorData(
          args.universe_domain
      )
      universe_descriptor_obj = universe_descriptor.UniverseDescriptor()
      _, is_deprecated_and_switched = (
          universe_descriptor_obj.UpdateDescriptorFromUniverseDomain(
              args.universe_domain
          )
      )
      # Avoid setting back the universe domain property if args.universe_domain
      # is deprecated.
      if not is_deprecated_and_switched:
        created_config.PersistProperty(
            'core', 'universe_domain', args.universe_domain
        )
      else:
        log.status.Print('Domain is switched to primary.')
      log.status.Print('Updated property [core/universe_domain].')

    return args.configuration_name



# -*- coding: utf-8 -*- # Lint as: python3
# Copyright 2020 Google Inc. All Rights Reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to deploy an Apigee API proxy to an environment."""


from googlecloudsdk.api_lib import apigee
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.apigee import defaults
from googlecloudsdk.command_lib.apigee import resource_args
from googlecloudsdk.core import log


class Deploy(base.DescribeCommand):
  """Deploy an API proxy to an environment."""

  detailed_help = {
      "DESCRIPTION":
          """\
   {description}

  `{command}` installs an API proxy revision in an Apigee runtime environment.

  By default, the API proxy's base path must not already be in use by a deployed
  proxy in the target environment. To allow Apigee to undeploy any conflicting
  API proxy as part of the deployment, use the `--override` command.

  Once a particular revision of an API proxy has been deployed, that revision
  can no longer be modified. Any updates to the API proxy must be saved as a new
  revision.
  """,
      "EXAMPLES":
          """\
  To deploy the latest revision of the API proxy named ``demo'' to the ``test''
  environment, given that the API proxy and environment's matching Cloud
  Platform project has been set in gcloud settings, run:

    $ {command} --environment=test --api=demo

  To deploy revision 3 of that proxy, owned by an organization named ``my-org'',
  run, and replace any conflicting deployment that might already exist, run:

    $ {command} 3 --organization=my-org --environment=test --api=demo --override

  To deploy that proxy and print the resulting deployment as a JSON object, run:

    $ {command} 3 --organization=my-org --environment=test --api=demo --format=json
  """
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        "--override",
        action="store_true",
        help=("Force the deployment of the new revision, overriding any " +
              "currently deployed revision that would conflict with it.\n\n" +
              "If an existing API proxy revision is deployed, set this flag " +
              "to ensure seamless deployment with zero downtime. In this " +
              "case, the existing revision remains deployed until the new " +
              "revision is fully deployed.\n\n" +
              "If unset, `{command}` will fail unless all conflicting API " +
              "proxies are first undeployed from the environment. To do " +
              "this, run `{parent_command} undeploy` on the conflicting " +
              "deployment."))

    help_text = {
        "api": ("API proxy to be deployed. To get a list of available API " +
                "proxies, run `{{parent_command}} list`."),
        "environment": ("Environment in which to deploy the API proxy. To " +
                        "get a list of available environments, run " +
                        "`{{grandparent_command}} environments list`."),
        "organization": ("Apigee organization of the proxy and environment. " +
                         "If unspecified, the Cloud Platform project's "
                         "associated organization will be used."),
    }
    fallthroughs = [defaults.GCPProductOrganizationFallthrough(),
                    defaults.StaticFallthrough("revision", "latest")]
    resource_args.AddSingleResourceArgument(
        parser,
        "organization.environment.api.revision",
        "API proxy revision to be deployed and environment in which to deploy "
        "it. Revisions can either be a positive revision number, or the "
        "special value ``latest'', which will deploy the latest revision of "
        "the API proxy. If revision is unspecified, the default is ``latest''.",
        fallthroughs=fallthroughs,
        help_texts=help_text)

  def Run(self, args):
    """Run the deploy command."""
    identifiers = args.CONCEPTS.revision.Parse().AsDict()
    if identifiers["revisionsId"] == "latest":
      latest_revision = apigee.APIsClient.Describe(identifiers)["revision"][-1]
      log.status.Print("Using current latest revision `%s`"%latest_revision)
      identifiers["revisionsId"] = latest_revision

    result = apigee.APIsClient.Deploy(identifiers, args.override)
    return result



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud access-context-manager policies quotas list` command."""


import dataclasses

from googlecloudsdk.api_lib.accesscontextmanager import levels as levels_api
from googlecloudsdk.api_lib.accesscontextmanager import zones as perimeters_api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.accesscontextmanager import policies


@dataclasses.dataclass
class Metric:
  title: str
  usage: int


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class ListPoliciesQuotas(base.ListCommand):
  """Lists the Quota Usage."""

  _API_VERSION = 'v1alpha'

  def GetPrimetersQuotaUsage(self, perimeters_to_display):
    """Returns service primeters quota usage.

    Args:
      perimeters_to_display: Response of ListServicePerimeters API
    """
    arguments = list(perimeters_to_display)

    service_primeters = len(arguments)
    protected_resources = 0
    ingress_rules = 0
    egress_rules = 0
    total_ingress_egress_attributes = self.GetTotalIngressEgressAttributes(
        arguments
    )

    for metric in arguments:
      configs = []
      if metric.status:
        configs.append(metric.status)
      if metric.spec:
        configs.append(metric.spec)
      for config in configs:
        protected_resources += len(config.resources)
        ingress_rules += len(config.ingressPolicies)
        egress_rules += len(config.egressPolicies)

    return [
        Metric('Service primeters', service_primeters),
        Metric('Protected resources', protected_resources),
        Metric('Ingress rules', ingress_rules),
        Metric('Egress rules', egress_rules),
        Metric(
            'Total ingress/egress attributes', total_ingress_egress_attributes
        ),
    ]

  def GetLevelsQuotaUsage(self, levels_to_display):
    """Returns levels quota usage, only counts basic access levels.

    Args:
      levels_to_display: Response of ListAccessLevels API
    """
    access_levels = 0
    for level in levels_to_display:
      if level.basic:
        access_levels += 1
    return [Metric('Access levels', access_levels)]

  def GetTotalIngressEgressAttributes(self, perimeters_to_display):
    """Returns total ingress/egress attributes quota usage.

    Args:
      perimeters_to_display: Response of ListServicePerimeters API
    """
    elements_count = 0
    for metric in perimeters_to_display:
      configs = []
      if metric.status:
        configs.append(metric.status)
      if metric.spec:
        configs.append(metric.spec)
      for config in configs:
        if config.ingressPolicies:
          for ingress_policy in config.ingressPolicies:
            elements_count += len(ingress_policy.ingressFrom.sources)
            elements_count += len(ingress_policy.ingressFrom.identities)
            elements_count += sum(
                len(o.methodSelectors)
                for o in ingress_policy.ingressTo.operations
            )
            elements_count += len(ingress_policy.ingressTo.resources)
        if config.egressPolicies:
          for egress_policy in config.egressPolicies:
            elements_count += len(egress_policy.egressFrom.identities)
            elements_count += sum(
                len(o.methodSelectors)
                for o in egress_policy.egressTo.operations
            )
            elements_count += len(egress_policy.egressTo.resources)
    return elements_count

  @staticmethod
  def Args(parser):
    policies.AddResourceArg(parser, 'to list the quota usage')
    base.URI_FLAG.RemoveFromParser(parser)
    parser.display_info.AddFormat('table(title, usage)')

  def Run(self, args):
    perimeters_client = perimeters_api.Client(version=self._API_VERSION)
    levels_client = levels_api.Client(version=self._API_VERSION)

    policy_ref = args.CONCEPTS.policy.Parse()

    levels_to_display = levels_client.List(policy_ref)
    perimeters_to_display = perimeters_client.List(policy_ref)

    primeters_quota_usage = self.GetPrimetersQuotaUsage(perimeters_to_display)
    levels_quota_usage = self.GetLevelsQuotaUsage(levels_to_display)

    return primeters_quota_usage + levels_quota_usage


detailed_help = {
    'brief': (
        'List the quota usage of a specific Access Context Manager policy.'
    ),
    'DESCRIPTION': (
        'List quota usage of a specific Access Context Manager policy,'
        ' also known as an access policy. Metrics include: Serivce perimeters,'
        ' Protected resources, Ingress rules, Egress rules, Access rules and'
        ' Total ingress/egress attributes. For access levels, this only counts'
        ' basic access levels.'
    ),
    'EXAMPLES': """
       To list the quota usage of a specific Access Context Manager policy:

       $ {command} POLICY

Sample output:

  ===
    TITLE                            USAGE
    Service primeters                1
    Protected resources              1
    Ingress rules                    1
    Egress rules                     1
    Total ingress/egress attributes  3
    Access levels                    1
""",
}

ListPoliciesQuotas.detailed_help = detailed_help



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to SCP to/from a Cloud TPU VM Node."""



from argcomplete.completers import FilesCompleter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags
from googlecloudsdk.command_lib.compute import ssh_utils
from googlecloudsdk.command_lib.compute.tpus.tpu_vm import ssh as tpu_ssh_utils
from googlecloudsdk.command_lib.util.ssh import ssh
from googlecloudsdk.core import properties


def AddSCPArgs(parser):
  """Additional flags and positional args to be passed to *scp(1)*."""
  parser.add_argument(
      '--scp-flag',
      action='append',
      help="""\
      Additional flags to be passed to *scp(1)*. This flag may be repeated.
      """)

  parser.add_argument(
      'sources',
      completer=FilesCompleter,
      help='Specifies the files to copy.',
      metavar='[[USER@]INSTANCE:]SRC',
      nargs='+')

  parser.add_argument(
      'destination',
      help='Specifies a destination for the source files.',
      metavar='[[USER@]INSTANCE:]DEST')

  parser.add_argument(
      '--recurse', action='store_true', help='Upload directories recursively.')

  parser.add_argument(
      '--compress', action='store_true', help='Enable compression.')


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Scp(base.Command):
  """Copy files to and from a Cloud TPU VM via SCP."""

  # IAP and Batching are not available for GA.
  _ENABLE_IAP = False
  _ENABLE_BATCHING = False

  @classmethod
  def Args(cls, parser):
    """Set up arguments for this command.

    Args:
      parser: An argparse.ArgumentParser.
    """
    ssh_utils.BaseSSHCLIHelper.Args(parser)
    tpu_ssh_utils.AddTPUSSHArgs(parser, cls._ENABLE_IAP, cls._ENABLE_BATCHING)
    AddSCPArgs(parser)
    flags.AddZoneFlag(parser, resource_type='tpu', operation_type='scp')

  def Run(self, args):
    dst = ssh.FileReference.FromPath(args.destination)
    srcs = [ssh.FileReference.FromPath(src) for src in args.sources]
    ssh.SCPCommand.Verify(srcs, dst, single_remote=True)

    remote = dst.remote or srcs[0].remote
    tpu_name = remote.host
    if not dst.remote:  # Make sure all remotes point to the same ref.
      for src in srcs:
        src.remote = remote

    username_requested = True
    if not remote.user:
      username_requested = False
      remote.user = ssh.GetDefaultSshUsername(warn_on_account_user=True)

    # If zone is not set, retrieve the one from the config.
    if args.zone is None:
      args.zone = properties.VALUES.compute.zone.Get(required=True)

    prepped_node = [None]
    tpu_ssh_utils.PrepareNodeForSCP(
        tpu_name,
        args,
        self.ReleaseTrack(),
        self._ENABLE_BATCHING,
        username_requested,
        prepped_node,
        0,
        srcs,
        dst,
        remote,
    )

    scp_batch_size = 1
    if self._ENABLE_BATCHING and prepped_node[0]:
      scp_batch_size = tpu_ssh_utils.ParseBatchSize(
          args.batch_size, len(prepped_node[0].worker_ips)
      )

    tpu_ssh_utils.SCPIntoPreppedNodes(prepped_node, args, scp_batch_size)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class ScpAlpha(Scp):
  """Copy files to and from a Cloud TPU VM via SCP (Alpha)."""
  _ENABLE_IAP = True
  _ENABLE_BATCHING = True


Scp.detailed_help = {
    'brief':
        'Copy files to and from a Cloud TPU VM via SCP.',
    'EXAMPLES':
        """
        To copy a file (for example, a text file in the local home directory) to
        a Cloud TPU VM, run:

            $ {command} ~/my-file my-tpu:

        To copy a file into all workers in a Cloud TPU VM, run:

            $ {command} ~/my-file my-tpu: --worker=all

        To copy a file from a Cloud TPU VM to the home directory of the local
        computer, run:

            $ {command} my-tpu:~/my-file ~/

        To copy all files in a folder to a Cloud TPU VM, run:

            $ {command} ~/my-folder/ my-tpu: --recurse
        """
}



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""ai-platform operations list command."""


from googlecloudsdk.api_lib.ml_engine import operations
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ml_engine import endpoint_util
from googlecloudsdk.command_lib.ml_engine import flags
from googlecloudsdk.command_lib.ml_engine import operations_util


def _AddListArgs(parser):
  list_format = """\
    table(
        name.basename(),
        metadata.operationType,
        done
    )
"""
  parser.display_info.AddFormat(list_format)
  flags.GetRegionArg().AddToParser(parser)


def _Run(args):
  with endpoint_util.MlEndpointOverrides(region=args.region):
    client = operations.OperationsClient()
    return operations_util.List(client)


@base.ReleaseTracks(base.ReleaseTrack.GA)
class List(base.ListCommand):
  """List existing AI Platform jobs."""

  @staticmethod
  def Args(parser):
    _AddListArgs(parser)

  def Run(self, args):
    return _Run(args)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class ListBeta(base.ListCommand):
  """List existing AI Platform jobs."""

  @staticmethod
  def Args(parser):
    _AddListArgs(parser)

  def Run(self, args):
    return _Run(args)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for Data Catalog tags commands."""


from googlecloudsdk.api_lib.data_catalog import tags
from googlecloudsdk.api_lib.data_catalog import tags_v1


def UpdateCreateTagRequestWithInput(unused_ref, args, request):
  del unused_ref

  client = tags.TagsClient()
  return client.ParseCreateTagArgsIntoRequest(args, request)


def UpdateCreateTagRequestWithInputV1(unused_ref, args, request):
  del unused_ref

  client = tags_v1.TagsClient()
  return client.ParseCreateTagArgsIntoRequest(args, request)


def UpdateUpdateTagRequestWithInput(unused_ref, args, request):
  del unused_ref

  client = tags.TagsClient()
  return client.ParseUpdateTagArgsIntoRequest(args, request)


def UpdateUpdateTagRequestWithInputV1(unused_ref, args, request):
  del unused_ref

  client = tags_v1.TagsClient()
  return client.ParseUpdateTagArgsIntoRequest(args, request)




"""Generated client library for cloudquotas version v1beta."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.cloudquotas.v1beta import cloudquotas_v1beta_messages as messages


class CloudquotasV1beta(base_api.BaseApiClient):
  """Generated client library for service cloudquotas version v1beta."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://cloudquotas.googleapis.com/'
  MTLS_BASE_URL = 'https://cloudquotas.mtls.googleapis.com/'

  _PACKAGE = 'cloudquotas'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1beta'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'CloudquotasV1beta'
  _URL_VERSION = 'v1beta'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new cloudquotas handle."""
    url = url or self.BASE_URL
    super(CloudquotasV1beta, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.folders_locations_quotaAdjusterSettings = self.FoldersLocationsQuotaAdjusterSettingsService(self)
    self.folders_locations_quotaPreferences = self.FoldersLocationsQuotaPreferencesService(self)
    self.folders_locations_services_quotaInfos = self.FoldersLocationsServicesQuotaInfosService(self)
    self.folders_locations_services = self.FoldersLocationsServicesService(self)
    self.folders_locations = self.FoldersLocationsService(self)
    self.folders = self.FoldersService(self)
    self.organizations_locations_gdcZones_gdcServices_gdcQuotaInfos = self.OrganizationsLocationsGdcZonesGdcServicesGdcQuotaInfosService(self)
    self.organizations_locations_gdcZones_gdcServices = self.OrganizationsLocationsGdcZonesGdcServicesService(self)
    self.organizations_locations_gdcZones = self.OrganizationsLocationsGdcZonesService(self)
    self.organizations_locations_quotaAdjusterSettings = self.OrganizationsLocationsQuotaAdjusterSettingsService(self)
    self.organizations_locations_quotaPreferences = self.OrganizationsLocationsQuotaPreferencesService(self)
    self.organizations_locations_services_quotaInfos = self.OrganizationsLocationsServicesQuotaInfosService(self)
    self.organizations_locations_services = self.OrganizationsLocationsServicesService(self)
    self.organizations_locations = self.OrganizationsLocationsService(self)
    self.organizations = self.OrganizationsService(self)
    self.projects_locations_gdcZones_gdcServices_gdcQuotaInfos = self.ProjectsLocationsGdcZonesGdcServicesGdcQuotaInfosService(self)
    self.projects_locations_gdcZones_gdcServices = self.ProjectsLocationsGdcZonesGdcServicesService(self)
    self.projects_locations_gdcZones = self.ProjectsLocationsGdcZonesService(self)
    self.projects_locations_quotaAdjusterSettings = self.ProjectsLocationsQuotaAdjusterSettingsService(self)
    self.projects_locations_quotaPreferences = self.ProjectsLocationsQuotaPreferencesService(self)
    self.projects_locations_services_quotaInfos = self.ProjectsLocationsServicesQuotaInfosService(self)
    self.projects_locations_services = self.ProjectsLocationsServicesService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class FoldersLocationsQuotaAdjusterSettingsService(base_api.BaseApiService):
    """Service class for the folders_locations_quotaAdjusterSettings resource."""

    _NAME = 'folders_locations_quotaAdjusterSettings'

    def __init__(self, client):
      super(CloudquotasV1beta.FoldersLocationsQuotaAdjusterSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def GetQuotaAdjusterSettings(self, request, global_params=None):
      r"""Gets the QuotaAdjusterSettings for the specified resource.

      Args:
        request: (CloudquotasFoldersLocationsQuotaAdjusterSettingsGetQuotaAdjusterSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaAdjusterSettings) The response message.
      """
      config = self.GetMethodConfig('GetQuotaAdjusterSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetQuotaAdjusterSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/folders/{foldersId}/locations/{locationsId}/quotaAdjusterSettings',
        http_method='GET',
        method_id='cloudquotas.folders.locations.quotaAdjusterSettings.getQuotaAdjusterSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudquotasFoldersLocationsQuotaAdjusterSettingsGetQuotaAdjusterSettingsRequest',
        response_type_name='QuotaAdjusterSettings',
        supports_download=False,
    )

    def UpdateQuotaAdjusterSettings(self, request, global_params=None):
      r"""Updates the QuotaAdjusterSettings for the specified resource.

      Args:
        request: (CloudquotasFoldersLocationsQuotaAdjusterSettingsUpdateQuotaAdjusterSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaAdjusterSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateQuotaAdjusterSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateQuotaAdjusterSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/folders/{foldersId}/locations/{locationsId}/quotaAdjusterSettings',
        http_method='PATCH',
        method_id='cloudquotas.folders.locations.quotaAdjusterSettings.updateQuotaAdjusterSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask', 'validateOnly'],
        relative_path='v1beta/{+name}',
        request_field='quotaAdjusterSettings',
        request_type_name='CloudquotasFoldersLocationsQuotaAdjusterSettingsUpdateQuotaAdjusterSettingsRequest',
        response_type_name='QuotaAdjusterSettings',
        supports_download=False,
    )

  class FoldersLocationsQuotaPreferencesService(base_api.BaseApiService):
    """Service class for the folders_locations_quotaPreferences resource."""

    _NAME = 'folders_locations_quotaPreferences'

    def __init__(self, client):
      super(CloudquotasV1beta.FoldersLocationsQuotaPreferencesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new QuotaPreference that declares the desired value for a quota.

      Args:
        request: (CloudquotasFoldersLocationsQuotaPreferencesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaPreference) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/folders/{foldersId}/locations/{locationsId}/quotaPreferences',
        http_method='POST',
        method_id='cloudquotas.folders.locations.quotaPreferences.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['ignoreSafetyChecks', 'quotaPreferenceId'],
        relative_path='v1beta/{+parent}/quotaPreferences',
        request_field='quotaPreference',
        request_type_name='CloudquotasFoldersLocationsQuotaPreferencesCreateRequest',
        response_type_name='QuotaPreference',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single QuotaPreference.

      Args:
        request: (CloudquotasFoldersLocationsQuotaPreferencesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaPreference) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/folders/{foldersId}/locations/{locationsId}/quotaPreferences/{quotaPreferencesId}',
        http_method='GET',
        method_id='cloudquotas.folders.locations.quotaPreferences.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudquotasFoldersLocationsQuotaPreferencesGetRequest',
        response_type_name='QuotaPreference',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists QuotaPreferences in a given project, folder or organization.

      Args:
        request: (CloudquotasFoldersLocationsQuotaPreferencesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListQuotaPreferencesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/folders/{foldersId}/locations/{locationsId}/quotaPreferences',
        http_method='GET',
        method_id='cloudquotas.folders.locations.quotaPreferences.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/quotaPreferences',
        request_field='',
        request_type_name='CloudquotasFoldersLocationsQuotaPreferencesListRequest',
        response_type_name='ListQuotaPreferencesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the parameters of a single QuotaPreference. It can updates the config in any states, not just the ones pending approval.

      Args:
        request: (CloudquotasFoldersLocationsQuotaPreferencesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaPreference) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/folders/{foldersId}/locations/{locationsId}/quotaPreferences/{quotaPreferencesId}',
        http_method='PATCH',
        method_id='cloudquotas.folders.locations.quotaPreferences.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'ignoreSafetyChecks', 'updateMask', 'validateOnly'],
        relative_path='v1beta/{+name}',
        request_field='quotaPreference',
        request_type_name='CloudquotasFoldersLocationsQuotaPreferencesPatchRequest',
        response_type_name='QuotaPreference',
        supports_download=False,
    )

  class FoldersLocationsServicesQuotaInfosService(base_api.BaseApiService):
    """Service class for the folders_locations_services_quotaInfos resource."""

    _NAME = 'folders_locations_services_quotaInfos'

    def __init__(self, client):
      super(CloudquotasV1beta.FoldersLocationsServicesQuotaInfosService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Retrieve the QuotaInfo of a quota for a project, folder or organization.

      Args:
        request: (CloudquotasFoldersLocationsServicesQuotaInfosGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaInfo) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/folders/{foldersId}/locations/{locationsId}/services/{servicesId}/quotaInfos/{quotaInfosId}',
        http_method='GET',
        method_id='cloudquotas.folders.locations.services.quotaInfos.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudquotasFoldersLocationsServicesQuotaInfosGetRequest',
        response_type_name='QuotaInfo',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists QuotaInfos of all quotas for a given project, folder or organization.

      Args:
        request: (CloudquotasFoldersLocationsServicesQuotaInfosListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListQuotaInfosResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/folders/{foldersId}/locations/{locationsId}/services/{servicesId}/quotaInfos',
        http_method='GET',
        method_id='cloudquotas.folders.locations.services.quotaInfos.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/quotaInfos',
        request_field='',
        request_type_name='CloudquotasFoldersLocationsServicesQuotaInfosListRequest',
        response_type_name='ListQuotaInfosResponse',
        supports_download=False,
    )

  class FoldersLocationsServicesService(base_api.BaseApiService):
    """Service class for the folders_locations_services resource."""

    _NAME = 'folders_locations_services'

    def __init__(self, client):
      super(CloudquotasV1beta.FoldersLocationsServicesService, self).__init__(client)
      self._upload_configs = {
          }

  class FoldersLocationsService(base_api.BaseApiService):
    """Service class for the folders_locations resource."""

    _NAME = 'folders_locations'

    def __init__(self, client):
      super(CloudquotasV1beta.FoldersLocationsService, self).__init__(client)
      self._upload_configs = {
          }

  class FoldersService(base_api.BaseApiService):
    """Service class for the folders resource."""

    _NAME = 'folders'

    def __init__(self, client):
      super(CloudquotasV1beta.FoldersService, self).__init__(client)
      self._upload_configs = {
          }

  class OrganizationsLocationsGdcZonesGdcServicesGdcQuotaInfosService(base_api.BaseApiService):
    """Service class for the organizations_locations_gdcZones_gdcServices_gdcQuotaInfos resource."""

    _NAME = 'organizations_locations_gdcZones_gdcServices_gdcQuotaInfos'

    def __init__(self, client):
      super(CloudquotasV1beta.OrganizationsLocationsGdcZonesGdcServicesGdcQuotaInfosService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets details of a single GdcQuotaInfo.

      Args:
        request: (CloudquotasOrganizationsLocationsGdcZonesGdcServicesGdcQuotaInfosGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GdcQuotaInfo) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/organizations/{organizationsId}/locations/{locationsId}/gdcZones/{gdcZonesId}/gdcServices/{gdcServicesId}/gdcQuotaInfos/{gdcQuotaInfosId}',
        http_method='GET',
        method_id='cloudquotas.organizations.locations.gdcZones.gdcServices.gdcQuotaInfos.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudquotasOrganizationsLocationsGdcZonesGdcServicesGdcQuotaInfosGetRequest',
        response_type_name='GdcQuotaInfo',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists GdcQuotaInfos in a given project and location.

      Args:
        request: (CloudquotasOrganizationsLocationsGdcZonesGdcServicesGdcQuotaInfosListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListGdcQuotaInfosResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/organizations/{organizationsId}/locations/{locationsId}/gdcZones/{gdcZonesId}/gdcServices/{gdcServicesId}/gdcQuotaInfos',
        http_method='GET',
        method_id='cloudquotas.organizations.locations.gdcZones.gdcServices.gdcQuotaInfos.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/gdcQuotaInfos',
        request_field='',
        request_type_name='CloudquotasOrganizationsLocationsGdcZonesGdcServicesGdcQuotaInfosListRequest',
        response_type_name='ListGdcQuotaInfosResponse',
        supports_download=False,
    )

  class OrganizationsLocationsGdcZonesGdcServicesService(base_api.BaseApiService):
    """Service class for the organizations_locations_gdcZones_gdcServices resource."""

    _NAME = 'organizations_locations_gdcZones_gdcServices'

    def __init__(self, client):
      super(CloudquotasV1beta.OrganizationsLocationsGdcZonesGdcServicesService, self).__init__(client)
      self._upload_configs = {
          }

  class OrganizationsLocationsGdcZonesService(base_api.BaseApiService):
    """Service class for the organizations_locations_gdcZones resource."""

    _NAME = 'organizations_locations_gdcZones'

    def __init__(self, client):
      super(CloudquotasV1beta.OrganizationsLocationsGdcZonesService, self).__init__(client)
      self._upload_configs = {
          }

  class OrganizationsLocationsQuotaAdjusterSettingsService(base_api.BaseApiService):
    """Service class for the organizations_locations_quotaAdjusterSettings resource."""

    _NAME = 'organizations_locations_quotaAdjusterSettings'

    def __init__(self, client):
      super(CloudquotasV1beta.OrganizationsLocationsQuotaAdjusterSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def GetQuotaAdjusterSettings(self, request, global_params=None):
      r"""Gets the QuotaAdjusterSettings for the specified resource.

      Args:
        request: (CloudquotasOrganizationsLocationsQuotaAdjusterSettingsGetQuotaAdjusterSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaAdjusterSettings) The response message.
      """
      config = self.GetMethodConfig('GetQuotaAdjusterSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetQuotaAdjusterSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/organizations/{organizationsId}/locations/{locationsId}/quotaAdjusterSettings',
        http_method='GET',
        method_id='cloudquotas.organizations.locations.quotaAdjusterSettings.getQuotaAdjusterSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudquotasOrganizationsLocationsQuotaAdjusterSettingsGetQuotaAdjusterSettingsRequest',
        response_type_name='QuotaAdjusterSettings',
        supports_download=False,
    )

    def UpdateQuotaAdjusterSettings(self, request, global_params=None):
      r"""Updates the QuotaAdjusterSettings for the specified resource.

      Args:
        request: (CloudquotasOrganizationsLocationsQuotaAdjusterSettingsUpdateQuotaAdjusterSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaAdjusterSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateQuotaAdjusterSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateQuotaAdjusterSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/organizations/{organizationsId}/locations/{locationsId}/quotaAdjusterSettings',
        http_method='PATCH',
        method_id='cloudquotas.organizations.locations.quotaAdjusterSettings.updateQuotaAdjusterSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask', 'validateOnly'],
        relative_path='v1beta/{+name}',
        request_field='quotaAdjusterSettings',
        request_type_name='CloudquotasOrganizationsLocationsQuotaAdjusterSettingsUpdateQuotaAdjusterSettingsRequest',
        response_type_name='QuotaAdjusterSettings',
        supports_download=False,
    )

  class OrganizationsLocationsQuotaPreferencesService(base_api.BaseApiService):
    """Service class for the organizations_locations_quotaPreferences resource."""

    _NAME = 'organizations_locations_quotaPreferences'

    def __init__(self, client):
      super(CloudquotasV1beta.OrganizationsLocationsQuotaPreferencesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new QuotaPreference that declares the desired value for a quota.

      Args:
        request: (CloudquotasOrganizationsLocationsQuotaPreferencesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaPreference) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/organizations/{organizationsId}/locations/{locationsId}/quotaPreferences',
        http_method='POST',
        method_id='cloudquotas.organizations.locations.quotaPreferences.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['ignoreSafetyChecks', 'quotaPreferenceId'],
        relative_path='v1beta/{+parent}/quotaPreferences',
        request_field='quotaPreference',
        request_type_name='CloudquotasOrganizationsLocationsQuotaPreferencesCreateRequest',
        response_type_name='QuotaPreference',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single QuotaPreference.

      Args:
        request: (CloudquotasOrganizationsLocationsQuotaPreferencesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaPreference) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/organizations/{organizationsId}/locations/{locationsId}/quotaPreferences/{quotaPreferencesId}',
        http_method='GET',
        method_id='cloudquotas.organizations.locations.quotaPreferences.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudquotasOrganizationsLocationsQuotaPreferencesGetRequest',
        response_type_name='QuotaPreference',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists QuotaPreferences in a given project, folder or organization.

      Args:
        request: (CloudquotasOrganizationsLocationsQuotaPreferencesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListQuotaPreferencesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/organizations/{organizationsId}/locations/{locationsId}/quotaPreferences',
        http_method='GET',
        method_id='cloudquotas.organizations.locations.quotaPreferences.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/quotaPreferences',
        request_field='',
        request_type_name='CloudquotasOrganizationsLocationsQuotaPreferencesListRequest',
        response_type_name='ListQuotaPreferencesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the parameters of a single QuotaPreference. It can updates the config in any states, not just the ones pending approval.

      Args:
        request: (CloudquotasOrganizationsLocationsQuotaPreferencesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaPreference) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/organizations/{organizationsId}/locations/{locationsId}/quotaPreferences/{quotaPreferencesId}',
        http_method='PATCH',
        method_id='cloudquotas.organizations.locations.quotaPreferences.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'ignoreSafetyChecks', 'updateMask', 'validateOnly'],
        relative_path='v1beta/{+name}',
        request_field='quotaPreference',
        request_type_name='CloudquotasOrganizationsLocationsQuotaPreferencesPatchRequest',
        response_type_name='QuotaPreference',
        supports_download=False,
    )

  class OrganizationsLocationsServicesQuotaInfosService(base_api.BaseApiService):
    """Service class for the organizations_locations_services_quotaInfos resource."""

    _NAME = 'organizations_locations_services_quotaInfos'

    def __init__(self, client):
      super(CloudquotasV1beta.OrganizationsLocationsServicesQuotaInfosService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Retrieve the QuotaInfo of a quota for a project, folder or organization.

      Args:
        request: (CloudquotasOrganizationsLocationsServicesQuotaInfosGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaInfo) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/organizations/{organizationsId}/locations/{locationsId}/services/{servicesId}/quotaInfos/{quotaInfosId}',
        http_method='GET',
        method_id='cloudquotas.organizations.locations.services.quotaInfos.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudquotasOrganizationsLocationsServicesQuotaInfosGetRequest',
        response_type_name='QuotaInfo',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists QuotaInfos of all quotas for a given project, folder or organization.

      Args:
        request: (CloudquotasOrganizationsLocationsServicesQuotaInfosListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListQuotaInfosResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/organizations/{organizationsId}/locations/{locationsId}/services/{servicesId}/quotaInfos',
        http_method='GET',
        method_id='cloudquotas.organizations.locations.services.quotaInfos.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/quotaInfos',
        request_field='',
        request_type_name='CloudquotasOrganizationsLocationsServicesQuotaInfosListRequest',
        response_type_name='ListQuotaInfosResponse',
        supports_download=False,
    )

  class OrganizationsLocationsServicesService(base_api.BaseApiService):
    """Service class for the organizations_locations_services resource."""

    _NAME = 'organizations_locations_services'

    def __init__(self, client):
      super(CloudquotasV1beta.OrganizationsLocationsServicesService, self).__init__(client)
      self._upload_configs = {
          }

  class OrganizationsLocationsService(base_api.BaseApiService):
    """Service class for the organizations_locations resource."""

    _NAME = 'organizations_locations'

    def __init__(self, client):
      super(CloudquotasV1beta.OrganizationsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

  class OrganizationsService(base_api.BaseApiService):
    """Service class for the organizations resource."""

    _NAME = 'organizations'

    def __init__(self, client):
      super(CloudquotasV1beta.OrganizationsService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsLocationsGdcZonesGdcServicesGdcQuotaInfosService(base_api.BaseApiService):
    """Service class for the projects_locations_gdcZones_gdcServices_gdcQuotaInfos resource."""

    _NAME = 'projects_locations_gdcZones_gdcServices_gdcQuotaInfos'

    def __init__(self, client):
      super(CloudquotasV1beta.ProjectsLocationsGdcZonesGdcServicesGdcQuotaInfosService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets details of a single GdcQuotaInfo.

      Args:
        request: (CloudquotasProjectsLocationsGdcZonesGdcServicesGdcQuotaInfosGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GdcQuotaInfo) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gdcZones/{gdcZonesId}/gdcServices/{gdcServicesId}/gdcQuotaInfos/{gdcQuotaInfosId}',
        http_method='GET',
        method_id='cloudquotas.projects.locations.gdcZones.gdcServices.gdcQuotaInfos.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudquotasProjectsLocationsGdcZonesGdcServicesGdcQuotaInfosGetRequest',
        response_type_name='GdcQuotaInfo',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists GdcQuotaInfos in a given project and location.

      Args:
        request: (CloudquotasProjectsLocationsGdcZonesGdcServicesGdcQuotaInfosListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListGdcQuotaInfosResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gdcZones/{gdcZonesId}/gdcServices/{gdcServicesId}/gdcQuotaInfos',
        http_method='GET',
        method_id='cloudquotas.projects.locations.gdcZones.gdcServices.gdcQuotaInfos.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/gdcQuotaInfos',
        request_field='',
        request_type_name='CloudquotasProjectsLocationsGdcZonesGdcServicesGdcQuotaInfosListRequest',
        response_type_name='ListGdcQuotaInfosResponse',
        supports_download=False,
    )

  class ProjectsLocationsGdcZonesGdcServicesService(base_api.BaseApiService):
    """Service class for the projects_locations_gdcZones_gdcServices resource."""

    _NAME = 'projects_locations_gdcZones_gdcServices'

    def __init__(self, client):
      super(CloudquotasV1beta.ProjectsLocationsGdcZonesGdcServicesService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsLocationsGdcZonesService(base_api.BaseApiService):
    """Service class for the projects_locations_gdcZones resource."""

    _NAME = 'projects_locations_gdcZones'

    def __init__(self, client):
      super(CloudquotasV1beta.ProjectsLocationsGdcZonesService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsLocationsQuotaAdjusterSettingsService(base_api.BaseApiService):
    """Service class for the projects_locations_quotaAdjusterSettings resource."""

    _NAME = 'projects_locations_quotaAdjusterSettings'

    def __init__(self, client):
      super(CloudquotasV1beta.ProjectsLocationsQuotaAdjusterSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def GetQuotaAdjusterSettings(self, request, global_params=None):
      r"""Gets the QuotaAdjusterSettings for the specified resource.

      Args:
        request: (CloudquotasProjectsLocationsQuotaAdjusterSettingsGetQuotaAdjusterSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaAdjusterSettings) The response message.
      """
      config = self.GetMethodConfig('GetQuotaAdjusterSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetQuotaAdjusterSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/quotaAdjusterSettings',
        http_method='GET',
        method_id='cloudquotas.projects.locations.quotaAdjusterSettings.getQuotaAdjusterSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudquotasProjectsLocationsQuotaAdjusterSettingsGetQuotaAdjusterSettingsRequest',
        response_type_name='QuotaAdjusterSettings',
        supports_download=False,
    )

    def UpdateQuotaAdjusterSettings(self, request, global_params=None):
      r"""Updates the QuotaAdjusterSettings for the specified resource.

      Args:
        request: (CloudquotasProjectsLocationsQuotaAdjusterSettingsUpdateQuotaAdjusterSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaAdjusterSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateQuotaAdjusterSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateQuotaAdjusterSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/quotaAdjusterSettings',
        http_method='PATCH',
        method_id='cloudquotas.projects.locations.quotaAdjusterSettings.updateQuotaAdjusterSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask', 'validateOnly'],
        relative_path='v1beta/{+name}',
        request_field='quotaAdjusterSettings',
        request_type_name='CloudquotasProjectsLocationsQuotaAdjusterSettingsUpdateQuotaAdjusterSettingsRequest',
        response_type_name='QuotaAdjusterSettings',
        supports_download=False,
    )

  class ProjectsLocationsQuotaPreferencesService(base_api.BaseApiService):
    """Service class for the projects_locations_quotaPreferences resource."""

    _NAME = 'projects_locations_quotaPreferences'

    def __init__(self, client):
      super(CloudquotasV1beta.ProjectsLocationsQuotaPreferencesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new QuotaPreference that declares the desired value for a quota.

      Args:
        request: (CloudquotasProjectsLocationsQuotaPreferencesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaPreference) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/quotaPreferences',
        http_method='POST',
        method_id='cloudquotas.projects.locations.quotaPreferences.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['ignoreSafetyChecks', 'quotaPreferenceId'],
        relative_path='v1beta/{+parent}/quotaPreferences',
        request_field='quotaPreference',
        request_type_name='CloudquotasProjectsLocationsQuotaPreferencesCreateRequest',
        response_type_name='QuotaPreference',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single QuotaPreference.

      Args:
        request: (CloudquotasProjectsLocationsQuotaPreferencesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaPreference) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/quotaPreferences/{quotaPreferencesId}',
        http_method='GET',
        method_id='cloudquotas.projects.locations.quotaPreferences.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudquotasProjectsLocationsQuotaPreferencesGetRequest',
        response_type_name='QuotaPreference',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists QuotaPreferences in a given project, folder or organization.

      Args:
        request: (CloudquotasProjectsLocationsQuotaPreferencesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListQuotaPreferencesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/quotaPreferences',
        http_method='GET',
        method_id='cloudquotas.projects.locations.quotaPreferences.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/quotaPreferences',
        request_field='',
        request_type_name='CloudquotasProjectsLocationsQuotaPreferencesListRequest',
        response_type_name='ListQuotaPreferencesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the parameters of a single QuotaPreference. It can updates the config in any states, not just the ones pending approval.

      Args:
        request: (CloudquotasProjectsLocationsQuotaPreferencesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaPreference) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/quotaPreferences/{quotaPreferencesId}',
        http_method='PATCH',
        method_id='cloudquotas.projects.locations.quotaPreferences.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'ignoreSafetyChecks', 'updateMask', 'validateOnly'],
        relative_path='v1beta/{+name}',
        request_field='quotaPreference',
        request_type_name='CloudquotasProjectsLocationsQuotaPreferencesPatchRequest',
        response_type_name='QuotaPreference',
        supports_download=False,
    )

  class ProjectsLocationsServicesQuotaInfosService(base_api.BaseApiService):
    """Service class for the projects_locations_services_quotaInfos resource."""

    _NAME = 'projects_locations_services_quotaInfos'

    def __init__(self, client):
      super(CloudquotasV1beta.ProjectsLocationsServicesQuotaInfosService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Retrieve the QuotaInfo of a quota for a project, folder or organization.

      Args:
        request: (CloudquotasProjectsLocationsServicesQuotaInfosGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QuotaInfo) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/quotaInfos/{quotaInfosId}',
        http_method='GET',
        method_id='cloudquotas.projects.locations.services.quotaInfos.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudquotasProjectsLocationsServicesQuotaInfosGetRequest',
        response_type_name='QuotaInfo',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists QuotaInfos of all quotas for a given project, folder or organization.

      Args:
        request: (CloudquotasProjectsLocationsServicesQuotaInfosListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListQuotaInfosResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/quotaInfos',
        http_method='GET',
        method_id='cloudquotas.projects.locations.services.quotaInfos.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/quotaInfos',
        request_field='',
        request_type_name='CloudquotasProjectsLocationsServicesQuotaInfosListRequest',
        response_type_name='ListQuotaInfosResponse',
        supports_download=False,
    )

  class ProjectsLocationsServicesService(base_api.BaseApiService):
    """Service class for the projects_locations_services resource."""

    _NAME = 'projects_locations_services'

    def __init__(self, client):
      super(CloudquotasV1beta.ProjectsLocationsServicesService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(CloudquotasV1beta.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(CloudquotasV1beta.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to create a new VPN gateway."""

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.vpn_gateways import vpn_gateways_utils
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import resource_manager_tags_utils
from googlecloudsdk.command_lib.compute.networks import flags as network_flags
from googlecloudsdk.command_lib.compute.vpn_gateways import flags


_VPN_GATEWAY_ARG = flags.GetVpnGatewayArgument()
_NETWORK_ARG = network_flags.NetworkArgumentForOtherResource("""\
  A reference to a network to which the VPN gateway is attached.
  """)


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
@base.UniverseCompatible
class Create(base.CreateCommand):
  """Create a new Compute Engine Highly Available VPN gateway.

  *{command}* creates a new Highly Available VPN gateway.

  Highly Available VPN Gateway provides a means to create a VPN solution with a
  higher availability SLA compared to Classic Target VPN Gateway.
  Highly Available VPN gateways are simply referred to as VPN gateways in the
  API documentation and gcloud commands.
  A VPN Gateway can reference one or more VPN tunnels that connect it to
  external VPN gateways or Cloud VPN Gateways.
  """

  detailed_help = {
      'EXAMPLES': (
          """\
          To create a VPN gateway, run:

              $ {command} my-vpn-gateway --region=us-central1 --network=default
          """
      )
  }
  _support_tagging_at_creation = True

  @classmethod
  def Args(cls, parser):
    """Set up arguments for this command."""
    parser.display_info.AddFormat(flags.DEFAULT_LIST_FORMAT)
    _NETWORK_ARG.AddArgument(parser)
    _VPN_GATEWAY_ARG.AddArgument(parser, operation_type='create')
    if cls._support_tagging_at_creation:
      parser.add_argument(
          '--resource-manager-tags',
          type=arg_parsers.ArgDict(),
          metavar='KEY=VALUE',
          help="""\
            A comma-separated list of Resource Manager tags to apply to the VPN gateway.
        """,
      )
    flags.GetDescriptionFlag().AddToParser(parser)
    flags.GetInterconnectAttachmentsFlag().AddToParser(parser)
    flags.GetStackType().AddToParser(parser)
    flags.GetGatewayIpVersion().AddToParser(parser)
    parser.display_info.AddCacheUpdater(flags.VpnGatewaysCompleter)

  def _Run(self, args):
    """Issues the request to create a new VPN gateway."""
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    helper = vpn_gateways_utils.VpnGatewayHelper(holder)
    messages = holder.client.messages
    vpn_gateway_ref = _VPN_GATEWAY_ARG.ResolveAsResource(args, holder.resources)
    network_ref = _NETWORK_ARG.ResolveAsResource(args, holder.resources)
    params = None
    vpn_interfaces_with_interconnect_attachments = None
    if args.interconnect_attachments is not None:
      vpn_interfaces_with_interconnect_attachments = (
          self._mapInterconnectAttachments(
              args,
              holder.resources,
              vpn_gateway_ref.region,
              vpn_gateway_ref.project,
          )
      )
    if self._support_tagging_at_creation:
      if args.resource_manager_tags is not None:
        params = self._CreateVpnGatewayParams(
            messages, args.resource_manager_tags
        )
    vpn_gateway_to_insert = helper.GetVpnGatewayForInsert(
        name=vpn_gateway_ref.Name(),
        description=args.description,
        network=network_ref.SelfLink(),
        vpn_interfaces_with_interconnect_attachments=vpn_interfaces_with_interconnect_attachments,
        stack_type=args.stack_type,
        gateway_ip_version=args.gateway_ip_version,
        support_tagging_at_creation=self._support_tagging_at_creation,
        params=params,
    )

    operation_ref = helper.Create(vpn_gateway_ref, vpn_gateway_to_insert)
    return helper.WaitForOperation(
        vpn_gateway_ref, operation_ref, 'Creating VPN Gateway'
    )

  def _mapInterconnectAttachments(self, args, resources, region, project):
    """Returns dict {interfaceId : interconnectAttachmentUrl} based on initial order of names in input interconnectAttachmentName and region and project of VPN Gateway.

    Args:
      args: Namespace, argparse.Namespace.
      resources: Generates resource references.
      region: VPN Gateway region.
      project: VPN Gateway project.
    """
    attachment_refs = args.interconnect_attachments
    if len(attachment_refs) == 1:
      return {
          0: flags.GetInterconnectAttachmentRef(
              resources, attachment_refs[0], region, project
          ).SelfLink()
      }
    else:
      return {
          0: flags.GetInterconnectAttachmentRef(
              resources, attachment_refs[0], region, project
          ).SelfLink(),
          1: flags.GetInterconnectAttachmentRef(
              resources, attachment_refs[1], region, project
          ).SelfLink(),
      }

  def _CreateVpnGatewayParams(self, messages, resource_manager_tags):
    resource_manager_tags_map = (
        resource_manager_tags_utils.GetResourceManagerTags(
            resource_manager_tags
        )
    )
    params = messages.VpnGatewayParams
    additional_properties = [
        params.ResourceManagerTagsValue.AdditionalProperty(key=key, value=value)
        for key, value in sorted(resource_manager_tags_map.items())
    ]
    return params(
        resourceManagerTags=params.ResourceManagerTagsValue(
            additionalProperties=additional_properties
        )
    )

  def Run(self, args):
    """See base.CreateCommand."""
    return self._Run(args)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class CreateBeta(Create):
  """Create a new Compute Engine Highly Available VPN gateway.

  *{command}* creates a new Highly Available VPN gateway.

  Highly Available VPN Gateway provides a means to create a VPN solution with a
  higher availability SLA compared to Classic Target VPN Gateway.
  Highly Available VPN gateways are simply referred to as VPN gateways in the
  API documentation and gcloud commands.
  A VPN Gateway can reference one or more VPN tunnels that connect it to
  external VPN gateways or Cloud VPN Gateways.
  """

  ROUTER_ARG = None
  INSTANCE_ARG = None

  _support_outer_vpn_ipv6 = True

  @classmethod
  def Args(cls, parser):
    """Set up arguments for this command."""
    super(CreateBeta, cls).Args(parser)

  def Run(self, args):
    """See base.CreateCommand."""
    return self._Run(args)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class CreateAlpha(Create):
  """Create a new Compute Engine Highly Available VPN gateway.

  *{command}* creates a new Highly Available VPN gateway.

  Highly Available VPN Gateway provides a means to create a VPN solution with a
  higher availability SLA compared to Classic Target VPN Gateway.
  Highly Available VPN gateways are simply referred to as VPN gateways in the
  API documentation and gcloud commands.
  A VPN Gateway can reference one or more VPN tunnels that connect it to
  external VPN gateways or Cloud VPN Gateways.
  """

  ROUTER_ARG = None
  INSTANCE_ARG = None

  @classmethod
  def Args(cls, parser):
    """Set up arguments for this command."""
    super(CreateAlpha, cls).Args(parser)

  def Run(self, args):
    """See base.CreateCommand."""
    return self._Run(args)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update public advertised prefix command."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import public_advertised_prefixes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.public_advertised_prefixes import flags


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class Update(base.UpdateCommand):
  r"""Updates a Compute Engine public advertised prefix.

  ## EXAMPLES

  To update a public advertised prefix:

    $ {command} my-pap --status=ptr-configured

  To announce a public advertised prefix:

    $ {command} my-pap --announce-prefix

  To withdraw a public advertised prefix:

    $ {command} my-pap --withdraw-prefix
  """

  @classmethod
  def Args(cls, parser):
    flags.MakePublicAdvertisedPrefixesArg().AddArgument(parser)
    announce_withdraw_parser = parser.add_mutually_exclusive_group(
        required=True)
    flags.AddUpdatePapArgsToParser(announce_withdraw_parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    messages = holder.client.messages
    resources = holder.resources

    pap_ref = flags.MakePublicAdvertisedPrefixesArg().ResolveAsResource(
        args,
        resources,
        scope_lister=compute_flags.GetDefaultScopeLister(holder.client))

    pap_client = public_advertised_prefixes.PublicAdvertisedPrefixesClient(
        client, messages, resources)

    if args.status is not None:
      return pap_client.Patch(pap_ref, status=args.status)
    if args.announce_prefix:
      return pap_client.Announce(pap_ref)
    if args.withdraw_prefix:
      return pap_client.Withdraw(pap_ref)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The main command group for Cloud Composer."""


from googlecloudsdk.calliope import base


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Composer(base.Group):
  """Create and manage Cloud Composer Environments.

  Cloud Composer is a managed Apache Airflow service that helps you create,
  schedule, monitor and manage workflows. Cloud Composer automation helps you
  create Airflow environments quickly and use Airflow-native tools, such as the
  powerful Airflow web interface and command line tools, so you can focus on
  your workflows and not your infrastructure.

  ## EXAMPLES

  To see how to create and manage environments, run:

      $ {command} environments --help

  To see how to manage long-running operations, run:

      $ {command} operations --help
  """

  category = base.DATA_ANALYTICS_CATEGORY

  def Filter(self, context, args):
    """Modify the context that will be given to this group's commands when run.

    Args:
      context: {str:object}, A set of key-value pairs that can be used for
          common initialization among commands.
      args: argparse.Namespace: The same namespace given to the corresponding
          .Run() invocation.

    Returns:
      The refined command context.
    """
    # TODO(b/190528955):  Determine if command group works with project number
    base.RequireProjectID(args)
    # The Composer API performs quota checking based on the resource project, so
    # user project overrides are not needed. The 'environments run' command
    # spawns a call to the Kubernetes Engine API, and the 'container' command
    # group also disables user project quota; removing this line will break
    # 'composer environments run.'
    base.DisableUserProjectQuota()

    return context



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command to update snapshot recycle bin policy."""

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import snapshot_recycle_bin_policy_utils
from googlecloudsdk.api_lib.compute.operations import poller
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.Hidden
@base.DefaultUniverseOnly
class Update(base.UpdateCommand):
  """Update the project's or organization's snapshot recycle bin policy."""

  detailed_help = {
      'EXAMPLES': """
      To set retention days for the default rule to 10 days for a project:

        $ {command} --set-rule=default --standard-snapshots-retention-duration-days=10

      To set retention days for the default rule to 6 days for organization 123456789:

        $ {command} --organization=123456789 --set-rule=default --standard-snapshots-retention-duration-days=6

      To set retention days for a tag-based rule '12345/env/prod' to 5 days for a project:

        $ {command} --set-rule='12345/env/prod' --standard-snapshots-retention-duration-days=5

      To remove rule '12345/env/prod' for a project:

        $ {command} --remove-rule='12345/env/prod'
      """,
  }

  @classmethod
  def Args(cls, parser):
    project_or_organization = parser.add_mutually_exclusive_group()
    project_or_organization.add_argument(
        '--organization',
        help='Organization ID to apply this policy to.',
    )
    project_or_organization.add_argument(
        '--project',
        help='Project ID to apply this policy to.',
    )
    action_group = parser.add_mutually_exclusive_group(required=True)

    set_rule_group = action_group.add_group()
    set_rule_group.add_argument(
        '--set-rule',
        metavar='RULE_KEY',
        help="""Key of the rule to set. Rule keys are 'default' or tag-based keys
        like '12345/env/prod'.""",
    )
    set_rule_group.add_argument(
        '--standard-snapshots-retention-duration-days',
        type=arg_parsers.BoundedInt(lower_bound=0),
        required=True,
        help="""Retention duration in days for standard snapshots.
        Must be provided with --set-rule.""",
    )

    action_group.add_argument(
        '--remove-rule',
        metavar='RULE_KEY',
        help="""Key of the rule to remove. Rule keys are 'default' or tag-based keys
        like '12345/env/prod'.""",
    )

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    messages = holder.client.messages

    rules_dict = {}

    if args.remove_rule:
      rules_dict[args.remove_rule] = None
      snapshot_recycle_bin_policy_utils.RegisterRulesValueCodec(messages)
    else:
      rule_key = args.set_rule
      retention_days = args.standard_snapshots_retention_duration_days
      rules_dict[rule_key] = messages.SnapshotRecycleBinPolicyRule(
          standardSnapshots=messages.SnapshotRecycleBinPolicyRuleRuleConfig(
              retentionDurationDays=retention_days
          )
      )

    updated_policy = messages.SnapshotRecycleBinPolicy(
        rules=messages.SnapshotRecycleBinPolicy.RulesValue(
            additionalProperties=[
                messages.SnapshotRecycleBinPolicy.RulesValue.AdditionalProperty(
                    key=k, value=v
                )
                for k, v in rules_dict.items()
            ]
        )
    )

    if args.organization:
      patch_request = (
          messages.ComputeOrganizationSnapshotRecycleBinPolicyPatchRequest(
              organization='organizations/' + args.organization,
              snapshotRecycleBinPolicy=updated_policy,
          )
      )
      service = client.apitools_client.organizationSnapshotRecycleBinPolicy
      log_message = 'Updated policy for organization [{0}].'.format(
          args.organization
      )
      result = service.Patch(patch_request)
      if result and result.name:
        log.status.Print(log_message)
      return None

    else:
      project = args.project or properties.VALUES.core.project.Get(
          required=True
      )
      patch_request = messages.ComputeSnapshotRecycleBinPolicyPatchRequest(
          project=project,
          snapshotRecycleBinPolicy=updated_policy,
      )
      service = client.apitools_client.snapshotRecycleBinPolicy
      params = {'project': project}
      policy_collection = 'compute.snapshotRecycleBinPolicy'
      log_message = 'Updated policy for project [{0}].'.format(project)

      results = client.MakeRequests(
          [(service, 'Patch', patch_request)], no_followup=True
      )
      if not results:
        log.status.Print(log_message)
        return None

      operation_ref = resources.REGISTRY.Parse(
          results[0].name,
          params=params,
          collection='compute.globalOperations',
      )
      policy_ref = holder.resources.Parse(
          None,
          params=params,
          collection=policy_collection,
      )
      operation_poller = poller.Poller(
          service,
          policy_ref,
      )
      waiter.WaitFor(
          operation_poller,
          operation_ref,
          'Waiting for snapshot recycle bin policy update to complete...',
      )
      log.status.Print(log_message)
